﻿using ProjectGasMachine.CollectionGenericObjects;
using ProjectGasMachine.Drawnings;

namespace ProjectGasMachine
{
    /// <summary>
    /// Форма работы с компанией и ее коллекцией
    /// </summary>
    public partial class FormGasMachineCollection : Form
    {
        /// <summary>
        /// Компания
        /// </summary>
        private AbstractCompany? _company = null;

        /// <summary>
        /// конструктор
        /// </summary>
        public FormGasMachineCollection()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Выбор компании
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (comboBoxSelectorCompany.Text)
            {
                case "Хранилище":
                    _company = new Autopark(pictureBox.Width, pictureBox.Height, new MassiveGenericObjects<DrawningMachine>());
                    break;
            }
        }
        /// <summary>
        /// Создание объекта класса-перемещения
        /// </summary>
        /// <param name="type"></param>
        private void CreateObject(string type)
        {
            if (_company == null)
            {
                return;
            }
            Random random = new();
            DrawningMachine drawningMachine;
            switch (type)
            {
                case nameof(DrawningMachine):
                    drawningMachine = new DrawningMachine(random.Next(100, 300), random.Next(1000, 3000), GetColor(random));
                    break;
                case nameof(DrawningGasMachine):
                    drawningMachine = new DrawningGasMachine(random.Next(100, 300), random.Next(1000, 3000), GetColor(random), GetColor(random),
                Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                    break;
                default:
                    return;
            }

            if (_company + drawningMachine != -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }
        /// <summary>
        /// Получение цыета
        /// </summary>
        /// <param name="random">генератор случайных чисел</param>
        /// <returns></returns>
        private static Color GetColor(Random random)
        {
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }

            return color;
        }

        /// <summary>
        /// добавление газовоза
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddGasMachine_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningGasMachine));

        /// <summary>
        /// добавление грузовика
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddMachine_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningMachine));

        /// <summary>
        /// удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveMachine_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
            {
                return;
            }


            if (MessageBox.Show("Удалить объект", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
            {
                return;
            }

            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        /// <summary>
        /// Передача объекта в другую форму
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGoToCheck_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            DrawningMachine? machine = null;
            int counter = 100;

            while (machine == null) 
            {
                machine = _company.GetRandomObject();
                counter--;
                if (counter <= 0) 
                {
                    break;
                }
            }

            if (machine == null) 
            {
                return;
            }

            FormGasMachine form = new()
            {
                SetMachine = machine
            };
            form.ShowDialog();
        }

        /// <summary>
        /// Перерисовка коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefresh_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            pictureBox.Image = _company.Show();
        }
    }
}
